// SPDX-License-Identifier: GPL-2.0-only
/*
 * Driver for the Epson RTC modules RX4901CE and RX8901CE
 * Copyright(C) SEIKO EPSON CORPORATION 2022. All rights reserved.
 *
 * Based on code from drivers/rtc/rtc-rx6110.c
 * Copyright(C) 2015 Pengutronix, Steffen Trumtrar <kernel@pengutronix.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#if 1				/* Set to 1 to enable debugging */
#define DEBUG
#include <linux/device.h>
#undef DEBUG
//#define EXTRA_DEBUGGING		/* Define for even more debugging */
//#define DUMP_REGISTERS		/* Define to dump RTC registers */
#endif

#include <linux/bcd.h>
#include <linux/i2c.h>
#include <linux/spi/spi.h>
#include <linux/module.h>
#include <linux/of_device.h>
#include <linux/rtc.h>
#include <linux/regmap.h>

/* RX4801/RX8901 Register definitions */
#define RX48901_REG_SEC		0x00
#define RX48901_REG_MIN		0x01
#define RX48901_REG_HOUR	0x02
#define RX48901_REG_WEEKDAY	0x03
#define RX48901_REG_DAY		0x04
#define RX48901_REG_MONTH	0x05
#define RX48901_REG_YEAR	0x06
#define RX48901_REG_ALM_MIN	0x07
#define RX48901_REG_ALM_HOUR	0x08
#define RX48901_REG_ALM_WEEKDAY	0x09
#define RX48901_REG_WTCNT_L	0x0A
#define RX48901_REG_WTCNT_M	0x0B
#define RX48901_REG_WTCNT_H	0x0C
#define RX48901_REG_TCTL	0x0D
#define RX48901_REG_INTF	0x0E
#define RX48901_REG_INTE	0x0F

#define RX48901_REG_BUF1_CFG2	0x29
#define RX48901_REG_PWSW_CFG	0x37

#define RX48901_REG_SRAM0	0x60
#define RX48901_REG_SRAM1	0x61
#define RX48901_REG_SRAM2	0x62
#define RX48901_REG_SRAM3	0x63
#define RX48901_REG_SRAM4	0x64
#define RX48901_REG_SRAM5	0x65
#define RX48901_REG_SRAM6	0x66
#define RX48901_REG_SRAM7	0x67
#define RX48901_REG_SRAM8	0x68
#define RX48901_REG_SRAM9	0x69
#define RX48901_REG_SRAMA	0x6A
#define RX48901_REG_SRAMB	0x6B
#define RX48901_REG_SRAMC	0x6C
#define RX48901_REG_SRAMD	0x6D
#define RX48901_REG_SRAME	0x6E
#define RX48901_REG_SRAMF	0x6F

/* Timer Control Register (0Dh) bit positions */
#define RX48901_BITS_TCTL_TSEL_1024HZ	0x00
#define RX48901_BITS_TCTL_TSEL_64HZ	0x01
#define RX48901_BITS_TCTL_TSEL_1HZ	0x02
#define RX48901_BITS_TCTL_TSEL_MIN	0x03
#define RX48901_BIT_TCTL_WADA		BIT(3)
#define RX48901_BIT_TCTL_TE		BIT(4)
#define RX48901_BIT_TCTL_USEL0		BIT(5)
#define RX48901_BITS_TCTL_FSEL_32768HZ	0x00
#define RX48901_BITS_TCTL_FSEL_1024HZ	0x40
#define RX48901_BITS_TCTL_FSEL_1HZ	0x80
#define RX48901_BITS_TCTL_FSEL_OFF	0xC0

/* Status Flag Register (0Eh) bit positions */
#define RX48901_BIT_INTF_VTMPLF		BIT(0)
#define RX48901_BIT_INTF_VLF		BIT(1)
#define RX48901_BIT_INTF_EVF		BIT(2)
#define RX48901_BIT_INTF_AF		BIT(3)
#define RX48901_BIT_INTF_TF		BIT(4)
#define RX48901_BIT_INTF_UF		BIT(5)
#define RX48901_BIT_INTF_OSCSTPF	BIT(6)
#define RX48901_BIT_INTF_PORF		BIT(7)

/* Interrupt Enable (0Fh) bit positions */
#define RX48901_BIT_INTE_STOP		BIT(0)
#define RX48901_BIT_INTE_EIE		BIT(2)
#define RX48901_BIT_INTE_AIE		BIT(3)
#define RX48901_BIT_INTE_TIE		BIT(4)
#define RX48901_BIT_INTE_UIE		BIT(5)
#define RX48901_BIT_INTE_CSEL_500MS	0x00
#define RX48901_BIT_INTE_CSEL_2SEC	0x40	/* default */
#define RX48901_BIT_INTE_CSEL_10SEC	0x80
#define RX48901_BIT_INTE_CSEL_30SEC	0xC0

/* Miscellaneous bit positions */
#define RX48901_BIT_ALARM_XWAE		BIT(7)
#define RX48901_BIT_B1C2_SRAMMOD	BIT(6)
#define RX48901_BIT_PWSW_SWSEL		0x04	/* default */
#define RX48901_BIT_PWSW_INIEN		BIT(6)
#define RX48901_BIT_PWSW_CHGEN		BIT(7)

enum {
	RTC_SEC = 0,
	RTC_MIN,
	RTC_HOUR,
	RTC_WDAY,
	RTC_DAY,
	RTC_MONTH,
	RTC_YEAR,
	RTC_TIME_SIZE
};

#define RX4901_DRIVER_NAME		"rx4901ce"
#define RX8901_DRIVER_NAME		"rx8901ce"

struct rx48901_data {
	struct device *dev;
	struct rtc_device *rtc;
	struct regmap *regmap;
	int irq;
};

static int rx48901_get_time(struct device *dev, struct rtc_time *tm)
{
	struct rx48901_data *rx48901 = dev_get_drvdata(dev);
	int ret;
	int flags;
	u8 data[RTC_TIME_SIZE];

	ret = regmap_read(rx48901->regmap, RX48901_REG_INTF, &flags);
	if (ret) {
		dev_err(dev, "%s: read error %d\n", __func__, ret);
		return ret;
	}

	/* check for VLF Flag (set at power-on) */
	if ((flags & RX48901_BIT_INTF_VLF)) {
		dev_warn_once(dev, "Voltage was low, data is invalid!\n");
		return -EINVAL;
	}

	/* read registers to date */
	ret = regmap_bulk_read(rx48901->regmap, RX48901_REG_SEC, data, ARRAY_SIZE(data));
	if (ret) {
		dev_err(dev, "%s: read error %d\n", __func__, ret);
		return ret;
	}

	tm->tm_sec  = bcd2bin(data[RTC_SEC] & 0x7F);
	tm->tm_min  = bcd2bin(data[RTC_MIN] & 0x7F);
	tm->tm_hour = bcd2bin(data[RTC_HOUR] & 0x3F);		/* only 24-hour clock */
	tm->tm_wday = ffs(data[RTC_WDAY] & 0x7F) - 1;
	tm->tm_mday = bcd2bin(data[RTC_DAY] & 0x3F);
	tm->tm_mon  = bcd2bin(data[RTC_MONTH] & 0x1F) - 1;
	tm->tm_year = bcd2bin(data[RTC_YEAR]) + 100;

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d year=%d wday=%d\n",
		__func__, tm->tm_sec, tm->tm_min, tm->tm_hour, tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900, tm->tm_wday);

	return 0;
}

static int rx48901_set_time(struct device *dev, struct rtc_time *tm)
{
	struct rx48901_data *rx48901 = dev_get_drvdata(dev);
	int ret;
	u8 data[RTC_TIME_SIZE];

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d year=%d wday=%d\n",
		__func__, tm->tm_sec, tm->tm_min, tm->tm_hour, tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900, tm->tm_wday);

	data[RTC_SEC]   = bin2bcd(tm->tm_sec);
	data[RTC_MIN]   = bin2bcd(tm->tm_min);
	data[RTC_HOUR]  = bin2bcd(tm->tm_hour);
	data[RTC_WDAY]  = BIT(tm->tm_wday);
	data[RTC_DAY]   = bin2bcd(tm->tm_mday);
	data[RTC_MONTH] = bin2bcd(tm->tm_mon + 1);
	data[RTC_YEAR]  = bin2bcd(tm->tm_year - 100);

	/* set STOP bit before changing clock/calendar */
	ret = regmap_set_bits(rx48901->regmap, RX48901_REG_INTE, RX48901_BIT_INTE_STOP);
	if (ret)
		return ret;

	ret = regmap_bulk_write(rx48901->regmap, RX48901_REG_SEC, data, ARRAY_SIZE(data));
	if (ret) {
		dev_err(dev, "%s: write error %d\n", __func__, ret);
		return ret;
	}

	/* clear STOP bit after changing clock/calendar */
	ret = regmap_clear_bits(rx48901->regmap, RX48901_REG_INTE, RX48901_BIT_INTE_STOP);
	if (ret)
		return ret;

	/* The time in the RTC is valid. Be sure to have VLF cleared. */
	ret = regmap_write(rx48901->regmap, RX48901_REG_INTF, ~(u8)RX48901_BIT_INTF_VLF);
	if (ret) {
		dev_err(dev, "%s: write error %d\n", __func__, ret);
		return ret;
	}

	return 0;
}

static irqreturn_t rx48901_irq(int irq, void *data)
{
	struct rx48901_data	*rx48901 = data;
	struct mutex		*lock = &rx48901->rtc->ops_lock;
	int flags;
	int ret;

	mutex_lock(lock);

	ret = regmap_read(rx48901->regmap, RX48901_REG_INTF, &flags);
	if (ret)
		goto out;

	dev_dbg(rx48901->dev, "%s: flags=%02X\n", __func__, flags);

	if (flags & RX48901_BIT_INTF_TF) {
		/* periodic */
		flags &= ~RX48901_BIT_INTF_TF;
		rtc_update_irq(rx48901->rtc, 1, RTC_PF | RTC_IRQF);
	}

	if (flags & RX48901_BIT_INTF_AF) {
		/* alarm */
		flags &= ~RX48901_BIT_INTF_AF;
		rtc_update_irq(rx48901->rtc, 1, RTC_AF | RTC_IRQF);
	}

	if (flags & RX48901_BIT_INTF_UF) {
		/* time update */
		flags &= ~RX48901_BIT_INTF_UF;
		rtc_update_irq(rx48901->rtc, 1, RTC_UF | RTC_IRQF);
	}

	regmap_write(rx48901->regmap, RX48901_REG_INTF, flags);
out:
	mutex_unlock(lock);

	return IRQ_HANDLED;
}

static int rx48901_read_alarm(struct device *dev, struct rtc_wkalrm *alrm)
{
	struct rx48901_data *rx48901 = dev_get_drvdata(dev);
	u8 data[3], flct[2];
	int ret;

	/* Read alarm registers. */
	ret = regmap_bulk_read(rx48901->regmap, RX48901_REG_ALM_MIN, data, ARRAY_SIZE(data));
	if (ret)
		return ret;
	dev_dbg(dev, "%s: alarm read %02X %02X %02X\n", __func__, data[0], data[1], data[2]);

	/* Read flag and control registers. */
	ret = regmap_bulk_read(rx48901->regmap, RX48901_REG_INTF, flct, ARRAY_SIZE(flct));
	if (ret)
		return ret;

	/* Report alarm 0 time assuming 24-hour and day-of-month modes. */
	alrm->time.tm_sec  = 0;
	alrm->time.tm_min  = bcd2bin(data[0] & 0x7F);
	alrm->time.tm_hour = bcd2bin(data[1] & 0x3F);
	if (!(data[2] & RX48901_BIT_ALARM_XWAE))
		alrm->time.tm_mday = bcd2bin(data[2] & 0x3F);

	alrm->enabled = !!(flct[1] & RX48901_BIT_INTE_AIE);
	alrm->pending = (flct[0] & RX48901_BIT_INTF_AF) && alrm->enabled;

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d wday=%d enabled=%d pending=%d\n",
		__func__, alrm->time.tm_sec, alrm->time.tm_min, alrm->time.tm_hour, alrm->time.tm_mday, alrm->time.tm_mon+1, alrm->time.tm_wday, alrm->enabled, alrm->pending);

	return 0;
}

static int rx48901_set_alarm(struct device *dev, struct rtc_wkalrm *alrm)
{
	struct rx48901_data *rx48901 = dev_get_drvdata(dev);
	u8 data[3], flct[2];
	int ret;

#ifdef EXTRA_DEBUGGING
	dev_dbg(dev, "set_alarm %s\n", alrm->enabled?"YES":"NO");
	dev_dbg(dev, "set_alarm UIE %d\n", rx48901->rtc->uie_rtctimer.enabled );
	dev_dbg(dev, "set_alarm AIR %d\n", rx48901->rtc->aie_timer.enabled );
#endif

	dev_dbg(dev, "%s: sec=%d min=%d hour=%d mday=%d mon=%d wday=%d enabled=%d pending=%d\n",
		__func__, alrm->time.tm_sec, alrm->time.tm_min, alrm->time.tm_hour, alrm->time.tm_mday, alrm->time.tm_mon+1, alrm->time.tm_wday, alrm->enabled, alrm->pending);

	/* Read flag and control registers. */
	ret = regmap_bulk_read(rx48901->regmap, RX48901_REG_INTF, flct, ARRAY_SIZE(flct));
	if (ret)
		return ret;

	if (flct[1] & (RX48901_BIT_INTE_AIE | RX48901_BIT_INTE_UIE)) {
		flct[1] &= ~(RX48901_BIT_INTE_AIE | RX48901_BIT_INTE_UIE);
		ret = regmap_write(rx48901->regmap, RX48901_REG_INTE, flct[1]);
		if (ret)
			return ret;
	}

	/* Hardware alarm precision is 1 minute! */
	data[0] = bin2bcd(alrm->time.tm_min);
	data[1] = bin2bcd(alrm->time.tm_hour);
	data[2] = bin2bcd(alrm->time.tm_mday);
	if (data[2] == 0)
		data[2] |= RX48901_BIT_ALARM_XWAE;		/* turn on XWAE bit to ignore day of month (no zero day) */
	dev_dbg(dev, "%s: alarm write %02X %02X %02X\n", __func__, data[0], data[1], data[2]);

	ret = regmap_bulk_write(rx48901->regmap, RX48901_REG_ALM_MIN, data, ARRAY_SIZE(data));
	if (ret)
		return ret;

	flct[0] &= ~(RX48901_BIT_INTF_AF | RX48901_BIT_INTF_UF);
	ret = regmap_write(rx48901->regmap, RX48901_REG_INTF, flct[0]);
	if (ret)
		return ret;

	if (!alrm->enabled)
		return 0;

	if ( rx48901->rtc->uie_rtctimer.enabled )
		flct[1] |= RX48901_BIT_INTE_UIE;					/* set update interrupt enable */
	if ( rx48901->rtc->aie_timer.enabled )
		flct[1] |= (RX48901_BIT_INTE_AIE | RX48901_BIT_INTE_UIE);		/* set alarm interrupt enable */

	return regmap_write(rx48901->regmap, RX48901_REG_INTE, flct[1]);
}

static int rx48901_alarm_irq_enable(struct device *dev, unsigned int enabled)
{
	struct rx48901_data *rx48901 = dev_get_drvdata(dev);
	int ctrl;
	int ret;

#ifdef EXTRA_DEBUGGING
	dev_dbg(dev, "alarm_irq_enable %s\n", enabled?"ON":"OFF");
	dev_dbg(dev, "alarm_irq_enable UIE %d\n", rx48901->rtc->uie_rtctimer.enabled );
	dev_dbg(dev, "alarm_irq_enable AIR %d\n", rx48901->rtc->aie_timer.enabled );
#endif

	ret = regmap_read(rx48901->regmap, RX48901_REG_INTE, &ctrl);
	if (ret)
		return ret;

	if (enabled) {
		if ( rx48901->rtc->uie_rtctimer.enabled )
			ctrl |= RX48901_BIT_INTE_UIE;				/* set update interrupt enable */
		if ( rx48901->rtc->aie_timer.enabled )
			ctrl |= (RX48901_BIT_INTE_AIE | RX48901_BIT_INTE_UIE);	/* set alarm interrupt enable */
	} else {
		if ( !rx48901->rtc->uie_rtctimer.enabled )
			ctrl &= ~RX48901_BIT_INTE_UIE;				/* clear update interrupt enable */
		if ( !rx48901->rtc->aie_timer.enabled )
			ctrl &= ~RX48901_BIT_INTE_AIE;				/* clear alarm interrupt enable */
	}

	ret = regmap_clear_bits(rx48901->regmap, RX48901_REG_INTF, RX48901_BIT_INTF_AF);
	if (ret)
		return ret;

	return regmap_write(rx48901->regmap, RX48901_REG_INTE, ctrl);
}

static int rx48901_nvmem_read(void *priv, unsigned int offset, void *val, size_t bytes)
{
	u8 *data = val;
	size_t read;
	int ret;
	while (bytes) {
		ret = regmap_write(priv, RX48901_REG_BUF1_CFG2, ((offset>>4) & 0x0F) | RX48901_BIT_B1C2_SRAMMOD);
		if (ret)
			return ret;
		read = 16 - (offset&0x0F);	/* limit read to this page */
		if (read > bytes)
			read = bytes;
		ret = regmap_bulk_read(priv, RX48901_REG_SRAM0 + (offset&0x0F), data, read);
		if (ret)
			return ret;
		data += read;
		offset += read;
		bytes -= read;
	}

	return 0;
}

static int rx48901_nvmem_write(void *priv, unsigned int offset, void *val, size_t bytes)
{
	u8 *data = val;
	size_t write;
	int ret;
	while (bytes) {
		ret = regmap_write(priv, RX48901_REG_BUF1_CFG2, ((offset>>4) & 0x0F) | RX48901_BIT_B1C2_SRAMMOD);
		if (ret)
			return ret;
		write = 16 - (offset&0x0F);	/* limit write to this page */
		if (write > bytes)
			write = bytes;
		ret = regmap_bulk_write(priv, RX48901_REG_SRAM0 + (offset&0x0F), data, write);
		if (ret)
			return ret;
		data += write;
		offset += write;
		bytes -= write;
	}

	return 0;
}

static const struct rtc_class_ops rx48901_rtc_ops = {
	.read_time	= rx48901_get_time,
	.set_time	= rx48901_set_time,
	.read_alarm	= rx48901_read_alarm,
	.set_alarm	= rx48901_set_alarm,
	.alarm_irq_enable = rx48901_alarm_irq_enable,
};

#ifdef DUMP_REGISTERS
static void rx48901_dump_bank(struct device *dev, int bank, u8 *data, int len)
{
	int i;
	int n;
	char out[128];

	n = sprintf(out, "Bank%d:", bank);
	for (i = 0; i < len; i++)
		n += sprintf(out+n, " %02X", data[i]);
	dev_dbg(dev, "%s\n", out);
}

static void rx48901_dump_registers(struct rx48901_data *rx48901)
{
	int ret;
	int bank;
	int len;
	u8 data[16];

	for (bank = 0; bank < 7; bank++) {
		switch (bank) {
		case 0:
			len = 16;
			ret = regmap_bulk_read(rx48901->regmap, 0x00, data, len);
			break;
		case 1:
			len = 9;
			ret = regmap_bulk_read(rx48901->regmap, 0x10, data, len);
			break;
		case 2:
			len = 16;
			ret = regmap_bulk_read(rx48901->regmap, 0x20, data, len);
			break;
		case 3:
			len = 5 + 6;
			ret = regmap_bulk_read(rx48901->regmap, 0x30, data, 5);
			if (!ret) ret = regmap_bulk_read(rx48901->regmap, 0x37, data+5, 6);
			break;
		case 4:
			len = 4 + 2 + 2;
			ret = regmap_bulk_read(rx48901->regmap, 0x41, data, 4);
			if (!ret) ret = regmap_bulk_read(rx48901->regmap, 0x46, data+4, 2);
			if (!ret) ret = regmap_bulk_read(rx48901->regmap, 0x4E, data+6, 2);
			break;
		case 5:
			len = 4;
			ret = regmap_bulk_read(rx48901->regmap, 0x51, data, len);
			break;
		case 6:
			len = 16;
			ret = regmap_bulk_read(rx48901->regmap, 0x60, data, len);
			break;
		}
		if (!ret)
			rx48901_dump_bank(rx48901->dev, bank, data, len);
	}
}
#else
  #define rx48901_dump_registers(r)
#endif

static int rx48901_power_init(struct rx48901_data *rx48901)
{
	struct device *dev = rx48901->dev;
	int ret;
	u32 chargeable;
	u8 pwsw = RX48901_BIT_PWSW_INIEN | RX48901_BIT_PWSW_SWSEL;

	if (!of_property_read_u32(dev->of_node, "charge-enable", &chargeable)) {
		if (chargeable == 1)
			pwsw |= RX48901_BIT_PWSW_CHGEN;
		else if (chargeable != 0)
			dev_warn(dev, "%s: Unsupported charge-enable value %u\n", __func__, chargeable);
	}

	ret = regmap_write(rx48901->regmap, RX48901_REG_PWSW_CFG, pwsw);
	if (ret)
		dev_err(dev, "%s: power write error %d\n", __func__, ret);

	dev_dbg(dev, "%s: Wrote %02X to power register %02X\n", __func__, pwsw, RX48901_REG_PWSW_CFG);
	return ret;
}

static int rx48901_init(struct rx48901_data *rx48901)
{
	struct device *dev = rx48901->dev;
	int flags;
	int ret;
	static const struct reg_sequence rx48901_default_regs[] = {
		{ RX48901_REG_ALM_MIN,		0x00 },
		{ RX48901_REG_ALM_HOUR,		0x00 },
		{ RX48901_REG_ALM_WEEKDAY,	0x00 },
		{ RX48901_REG_TCTL,		RX48901_BITS_TCTL_TSEL_1HZ | RX48901_BIT_TCTL_WADA | RX48901_BITS_TCTL_FSEL_OFF },	/* USEL0=0, TE=0 */
		{ RX48901_REG_BUF1_CFG2,	RX48901_BIT_B1C2_SRAMMOD },
	};

	rx48901_dump_registers(rx48901);

	ret = regmap_register_patch(rx48901->regmap, rx48901_default_regs, ARRAY_SIZE(rx48901_default_regs));
	if (ret) {
		dev_err(dev, "%s: patch write error %d\n", __func__, ret);
		return ret;
	}

	ret = rx48901_power_init(rx48901);
	if (ret)
		return ret;

	ret = regmap_read(rx48901->regmap, RX48901_REG_INTF, &flags);
	if (ret) {
		dev_err(dev, "%s: flag read error %d\n", __func__, ret);
		return ret;
	}

#ifdef EXTRA_DEBUGGING
	if (flags & RX48901_BIT_INTF_PORF)
		dev_warn(dev, "Power on reset was detected.\n");
	if (flags & RX48901_BIT_INTF_OSCSTPF)
		dev_warn(dev, "Oscillation stop was detected.\n");
	if ((flags & RX48901_BIT_INTF_VTMPLF))
		dev_warn(dev, "Voltage drop was detected.\n");
#endif

	/* clear all flags BUT VLF */
	flags &= ~(RX48901_BIT_INTF_VTMPLF | RX48901_BIT_INTF_EVF | RX48901_BIT_INTF_AF | RX48901_BIT_INTF_TF | RX48901_BIT_INTF_UF | RX48901_BIT_INTF_OSCSTPF | RX48901_BIT_INTF_PORF);
	ret = regmap_write(rx48901->regmap, RX48901_REG_INTF, flags);
	if (ret) {
		dev_err(dev, "%s: flag write error %d\n", __func__, ret);
		return ret;
	}

	return ret;
}

static int rx48901_probe(struct rx48901_data *rx48901, struct device *dev)
{
	int err;
	struct nvmem_config nvmem_cfg = {
		.name = "rx48901ce_nvmem",
		.word_size = 1,
		.stride = 1,
		.size = (RX48901_REG_SRAMF - RX48901_REG_SRAM0 + 1)*16,
		.reg_read = rx48901_nvmem_read,
		.reg_write = rx48901_nvmem_write,
		.type = NVMEM_TYPE_BATTERY_BACKED,
	};

	rx48901->rtc = devm_rtc_allocate_device(dev);
	if (IS_ERR(rx48901->rtc)) {
		dev_err(dev, "%s: rtc device allocation failed!\n", __func__);
		return PTR_ERR(rx48901->rtc);
	}

	err = rx48901_init(rx48901);
	if (err)
		return err;

	if (rx48901->irq > 0) {
		err = devm_request_threaded_irq(dev, rx48901->irq, NULL, rx48901_irq, IRQF_SHARED | IRQF_ONESHOT, dev_name(dev), rx48901);
		if (err) {
			rx48901->irq = 0;
			device_set_wakeup_capable(dev, false);
			dev_warn(dev, "%s: unable to request IRQ!\n", __func__);
		} else {
			device_set_wakeup_capable(dev, true);
			dev_dbg(dev, "%s: IRQ=%d\n", __func__, rx48901->irq);
		}
	}

	rx48901->rtc->ops = &rx48901_rtc_ops;
	rx48901->rtc->range_min = RTC_TIMESTAMP_BEGIN_2000;
	rx48901->rtc->range_max = RTC_TIMESTAMP_END_2099;
	rx48901->rtc->max_user_freq = 1;

	err = devm_rtc_register_device(rx48901->rtc);
	if (err) {
		dev_err(dev, "%s: rtc device registration failed!\n", __func__);
		return err;
	}

	nvmem_cfg.priv = rx48901->regmap;
	devm_rtc_nvmem_register(rx48901->rtc, &nvmem_cfg);

	return 0;
}

#ifdef CONFIG_OF
static const struct of_device_id rx48901_of_match[] = {
	{ .compatible = "epson,rx4901ce" },
	{ .compatible = "epson,rx8901ce" },
	{ },
};
MODULE_DEVICE_TABLE(of, rx48901_of_match);
#endif

#if IS_ENABLED(CONFIG_SPI_MASTER)
static struct regmap_config rx4901_regmap_config = {
	.reg_bits = 8,
	.val_bits = 8,
	.max_register = RX48901_REG_SRAMF,
	.read_flag_mask = 0x80,
};

static int rx4901_probe(struct spi_device *spi)
{
	struct device		*dev = &spi->dev;
	struct rx48901_data	*rx48901;
	int			err;

	spi->mode |= SPI_MODE_3;
/*	spi->mode ^= SPI_CS_HIGH; */	/* CS active high must be set in device tree! */
	spi->bits_per_word = 8;
	spi->max_speed_hz = 2000000;
	err = spi_setup(spi);
	if (err) {
		dev_err(dev, "%s: SPI setup failed!\n", __func__);
		return err;
	}

	dev_dbg(dev, "SPI settings: bits_per_word=%d, max_speed_hz=%d, mode=%Xh\n",
		 spi->bits_per_word, spi->max_speed_hz, spi->mode);

	rx48901 = devm_kzalloc(dev, sizeof(*rx48901), GFP_KERNEL);
	if (!rx48901)
		return -ENOMEM;

	rx48901->dev = dev;
	rx48901->regmap = devm_regmap_init_spi(spi, &rx4901_regmap_config);
	if (IS_ERR(rx48901->regmap)) {
		dev_err(dev, "%s: regmap init failed!\n", __func__);
		return PTR_ERR(rx48901->regmap);
	}

	rx48901->irq = spi->irq;
	spi_set_drvdata(spi, rx48901);

	return rx48901_probe(rx48901, dev);
}

static const struct spi_device_id rx4901_id[] = {
	{ "rx4901ce", 0 },
/*	{ "rx8901ce", 1 }, */
	{ }
};
MODULE_DEVICE_TABLE(spi, rx4901_id);

static struct spi_driver rx4901_driver = {
	.driver = {
		.name = RX4901_DRIVER_NAME,
		.of_match_table = of_match_ptr(rx48901_of_match),
	},
	.probe		= rx4901_probe,
	.id_table	= rx4901_id,
};

static int rx4901_register_driver(void)
{
	return spi_register_driver(&rx4901_driver);
}

static void rx4901_unregister_driver(void)
{
	spi_unregister_driver(&rx4901_driver);
}
#else
static int rx4901_register_driver(void)
{
	return 0;
}

static void rx4901_unregister_driver(void)
{
}
#endif /* CONFIG_SPI_MASTER */

#if IS_ENABLED(CONFIG_I2C)
static struct regmap_config rx8901_regmap_config = {
	.reg_bits = 8,
	.val_bits = 8,
	.max_register = RX48901_REG_SRAMF,
	.read_flag_mask = 0x80,
};

static int rx8901_probe(struct i2c_client *client)
{
	struct device		*dev = &client->dev;
	struct rx48901_data	*rx48901;

	if (!i2c_check_functionality(client->adapter, I2C_FUNC_I2C)) {
		dev_err(&client->adapter->dev, "%s: Adapter doesn't support required I2C functionality!\n", __func__);
		return -ENODEV;
	}

	rx48901 = devm_kzalloc(dev, sizeof(*rx48901), GFP_KERNEL);
	if (!rx48901)
		return -ENOMEM;

	rx48901->dev = dev;
	rx48901->regmap = devm_regmap_init_i2c(client, &rx8901_regmap_config);
	if (IS_ERR(rx48901->regmap)) {
		dev_err(dev, "%s: regmap init failed!\n", __func__);
		return PTR_ERR(rx48901->regmap);
	}

	rx48901->irq = client->irq;
	i2c_set_clientdata(client, rx48901);

	return rx48901_probe(rx48901, dev);
}

static const struct i2c_device_id rx8901_id[] = {
	{ "rx8901ce", 0 },
	{ }
};
MODULE_DEVICE_TABLE(i2c, rx8901_id);

static struct i2c_driver rx8901_driver = {
	.driver = {
		.name = RX8901_DRIVER_NAME,
		.of_match_table = of_match_ptr(rx48901_of_match),
	},
	.probe_new	= rx8901_probe,
	.id_table	= rx8901_id,
};

static int rx8901_register_driver(void)
{
	return i2c_add_driver(&rx8901_driver);
}

static void rx8901_unregister_driver(void)
{
	i2c_del_driver(&rx8901_driver);
}
#else
static int rx8901_register_driver(void)
{
	return 0;
}

static void rx8901_unregister_driver(void)
{
}
#endif /* CONFIG_I2C */

static int __init rx48901_module_init(void)
{
	int ret;

	ret = rx4901_register_driver();
	if (ret)
		return ret;

	ret = rx8901_register_driver();
	if (ret)
		rx4901_unregister_driver();

	return ret;
}
module_init(rx48901_module_init);

static void __exit rx48901_module_exit(void)
{
	rx4901_unregister_driver();
	rx8901_unregister_driver();
}
module_exit(rx48901_module_exit);

MODULE_AUTHOR("Epson VDC <vdctech_info@ea.epson.com>");
MODULE_DESCRIPTION("RX4901/RX8901 CE RTC driver");
MODULE_LICENSE("GPL");
